%% Setup comparison methods

addpath('DOPBoxV1-8/DOPbox');
addpath('grad2SurfV1-0/grad2Surf');
addpath('normal_integration/Toolbox/');

options.structure = 'square';
options.precond = 'none';
options.solver = 'direct';

options.regularisation='smoothness';
options.lambda = 2;

%% Generate data

options.cameramodel = 'orthographic';

[Z_gt,dZdx_gt,dZdy_gt] = FuncAndDerivs();
mask = true(size(Z_gt));
sigma = 0.1;
dZdx = dZdx_gt+randn(size(Z_gt)).*sigma;
dZdy = dZdy_gt+randn(size(Z_gt)).*sigma;
evalmask = mask;

N = cat(3,-dZdx,-dZdy,ones(size(dZdx)));
N = EWnorm(N);

options.z0 = zeros(size(mask));

%% Proposed methods

npix = sum(mask(:));

figure;
subplot(2,4,1)
if strcmp(options.cameramodel,'perspective')
    [X,Y,~]=depth2pointcloud(Z_gt,fx,fy,cx,cy,Z_gt,mask);
    surf(X,-Y,-Z_gt, 'FaceColor', [1 1 1], 'EdgeColor', 'none'); light; axis equal
else
    surf(Z_gt, 'FaceColor', [1 1 1], 'EdgeColor', 'none'); light; axis equal
end
title('Ground truth')
drawnow

nOrders = {2,3,4};
nSizes = {7,7,7};
count = 1;
for i=1:length(nOrders)
    [Dx,Dy,options.S] = gradMatricesSavGol(mask,nOrders{i},nSizes{i});
    Z_est = surfaceFromGradient(Dx,Dy,dZdx,dZdy,mask,options);
    
    count = count+1;
    subplot(2,5,count)
    if strcmp(options.cameramodel,'perspective')
        disp(['Order ' num2str(nOrders{i}) ' RMSE = ' num2str(RMSE(Z_gt,Z_est,evalmask,false,true))]);
        [X,Y,Z_est]=depth2pointcloud(Z_est,fx,fy,cx,cy,Z_gt,mask);
        surf(X,-Y,-Z_est, 'FaceColor', [1 1 1], 'EdgeColor', 'none'); light; axis equal
    else
        disp(['Order ' num2str(nOrders{i}) ' RMSE = ' num2str(RMSE(Z_gt,Z_est,evalmask,true))]);
        surf(Z_est, 'FaceColor', [1 1 1], 'EdgeColor', 'none'); light; axis equal
    end
    title(['Order ' num2str(nOrders{i})])
    drawnow
end

%% Harker methods

p = -dZdy;
q = dZdx;

N=11;
Z_est = g2s( q, p, (1:size(mask,2))', (1:size(mask,1))', N );
Z_est(~mask) = NaN;
count = count +1;
subplot(2,5,count)
if strcmp(options.cameramodel,'perspective')
    disp(['Harker LSQ RMSE = ' num2str(RMSE(Z_gt,exp(Z_est),evalmask,false,true))]);
    [X,Y,Z_est]=depth2pointcloud(exp(Z_est),fx,fy,cx,cy,Z_gt,mask);
    surf(X,-Y,-Z_est, 'FaceColor', [1 1 1], 'EdgeColor', 'none'); light; axis equal
else
    disp(['Harker LSQ RMSE = ' num2str(RMSE(Z_gt,Z_est,evalmask,true))]);
    surf(Z_est, 'FaceColor', [1 1 1], 'EdgeColor', 'none'); light; axis equal
end
title('Harker LSQ')
drawnow

m=size(mask,1);
n=size(mask,2);
deg = 0 ;
Z0 = zeros(m,n) ;
noLambdas = 100 ;
[ Z_est, ~, ~, ~ ] = g2sTikhonovStd( q, p, (1:size(mask,2))', (1:size(mask,1))', N, noLambdas, Z0 ) ;
Z_est(~mask) = NaN;

count = count +1;
subplot(2,5,count)
if strcmp(options.cameramodel,'perspective')
    disp(['Harker Tik RMSE = ' num2str(RMSE(Z_gt,exp(Z_est),evalmask,false,true))]);
    [X,Y,Z_est]=depth2pointcloud(exp(Z_est),fx,fy,cx,cy,Z_gt,mask);
    surf(X,-Y,-Z_est, 'FaceColor', [1 1 1], 'EdgeColor', 'none'); light; axis equal
else
    disp(['Harker Tik RMSE = ' num2str(RMSE(Z_gt,Z_est,evalmask,true))]);
    surf(Z_est, 'FaceColor', [1 1 1], 'EdgeColor', 'none'); light; axis equal
end
title('Harker Tik')
drawnow

%% Quadratic and anisotropic

lambda = 1e-6*ones(size(p)); % Uniform field of weights (nrows x ncols)
z0 = zeros(size(p)); % Null depth prior (nrows x ncols)
solver = 'pcg'; % Solver ('pcg' means conjugate gradient, 'direct' means backslash i.e. sparse Cholesky)
precond = 'ichol'; % Preconditioner ('none' means no preconditioning, 'ichol' means incomplete Cholesky, 'CMG' means conjugate combinatorial multigrid -- the latter is fastest, but it need being installed, see README)

Z_est = smooth_integration(p,q,mask,lambda,z0,solver,precond);
zinit=Z_est; % To initialise aniso
count = count +1;
subplot(2,5,count)
if strcmp(options.cameramodel,'perspective')
    disp(['Quadratic RMSE = ' num2str(RMSE(Z_gt,exp(Z_est),evalmask,false,true))]);
    [X,Y,Z_est]=depth2pointcloud(exp(Z_est),fx,fy,cx,cy,Z_gt,mask);
    surf(X,-Y,-Z_est, 'FaceColor', [1 1 1], 'EdgeColor', 'none'); light; axis equal
else
    disp(['Quadratic RMSE = ' num2str(RMSE(Z_gt,Z_est,evalmask,true))]);
    surf(Z_est, 'FaceColor', [1 1 1], 'EdgeColor', 'none'); light; axis equal
end
title('Quadratic')
drawnow

% TV
alpha = 0.1; % Descent stepsize (influences speed)
tol = 1e-5; % Stopping criterion
maxit = 1000; % Stopping criterion

Z_est = tv_integration(p,q,mask,lambda,z0,alpha,maxit,tol,zinit);
count = count +1;
subplot(2,5,count)
if strcmp(options.cameramodel,'perspective')
    disp(['TV RMSE = ' num2str(RMSE(Z_gt,exp(Z_est),evalmask,false,true))]);
    [X,Y,Z_est]=depth2pointcloud(exp(Z_est),fx,fy,cx,cy,Z_gt,mask);
    surf(X,-Y,-Z_est, 'FaceColor', [1 1 1], 'EdgeColor', 'none'); light; axis equal
else
    disp(['TV RMSE = ' num2str(RMSE(Z_gt,Z_est,evalmask,true))]);
    surf(Z_est, 'FaceColor', [1 1 1], 'EdgeColor', 'none'); light; axis equal
end
title('TV')
drawnow

%% Finite difference comparison

% Backward
[ Dx,Dy,maskmod,~ ] = gradMatrices( mask,'Backward' );
if ~isequal(mask,maskmod)
    mask = maskmod;
    evalmask = maskmod;
end
options.S = LaplacianMatrix(mask);
Z_est = surfaceFromGradient(Dx,Dy,dZdx,dZdy,mask,options);
count = count +1;
subplot(2,5,count)
if strcmp(options.cameramodel,'perspective')
    disp(['Backward difference RMSE = ' num2str(RMSE(Z_gt,Z_est,evalmask,false,true))]);
    [X,Y,Z_est]=depth2pointcloud(Z_est,fx,fy,cx,cy,Z_gt,mask);
    surf(X,-Y,-Z_est, 'FaceColor', [1 1 1], 'EdgeColor', 'none'); light; axis equal
else
    disp(['Backward difference RMSE = ' num2str(RMSE(Z_gt,Z_est,evalmask,true))]);
    surf(Z_est, 'FaceColor', [1 1 1], 'EdgeColor', 'none'); light; axis equal
end
title('Backward finite difference')
drawnow

% Smoothed central
[ Dx,Dy,maskmod,~ ] = gradMatrices( mask,'SmoothedCentral' );
if ~isequal(mask,maskmod)
    mask = maskmod;
    evalmask = maskmod;
end
options.S = LaplacianMatrix(mask);
Z_est = surfaceFromGradient(Dx,Dy,dZdx,dZdy,mask,options);
count = count +1;
subplot(2,5,count)
if strcmp(options.cameramodel,'perspective')
    disp(['Smoothed central difference RMSE = ' num2str(RMSE(Z_gt,Z_est,evalmask,false,true))]);
    [X,Y,Z_est]=depth2pointcloud(Z_est,fx,fy,cx,cy,Z_gt,mask);
    surf(X,-Y,-Z_est, 'FaceColor', [1 1 1], 'EdgeColor', 'none'); light; axis equal
else
    disp(['Smoothed central difference RMSE = ' num2str(RMSE(Z_gt,Z_est,evalmask,true))]);
    surf(Z_est, 'FaceColor', [1 1 1], 'EdgeColor', 'none'); light; axis equal
end
title('Smoothed central difference')
drawnow

%%

figure;
subplot(1,2,1)
imshow(dZdx,[])
title('Noisy x gradient')
subplot(1,2,2)
imshow(dZdy,[])
title('Noisy y gradient')
drawnow